use domo_lib::node::Node;
use domo_proto::data_types::DataType;
use domo_proto::identifier::Identifier;
use log::{debug, error, info, trace, warn};
use std::{env, io};
use std::cell::RefCell;

pub mod config;

thread_local! {
    static CONFIG: RefCell<config::Config> = RefCell::new(config::Config::from_path(
        env::var("DOMO_CONFIG_PATH")
            .unwrap_or("config.toml".into())
    ).unwrap_or({ info!("Using default config..."); config::Config::default() }))
}

fn main() -> io::Result<()> {
    setup_logging().expect("could not setup logging");

    debug!("Building tokio runtime...");

    let runtime = tokio::runtime::Builder::new_multi_thread()
        .enable_all()
        .build()
        .unwrap();

    debug!("Built tokio runtime with all features...");

    info!(
        "Staring {} node as 0x{}!",
        CONFIG.with_borrow(|c| c.node.node_type.clone()),
        CONFIG.with_borrow(|c| c.node.node_id.clone())
    );


    runtime.block_on(async {
        let self_node = Node::new(
            Identifier::from(
                CONFIG.with_borrow(|c| 
                    hex::decode(&c.node.node_id)
                    .expect("Could not decode node_id")
                )
            )
        );

    });

    Ok(())
}

fn setup_logging() -> Result<(), fern::InitError> {
    fern::Dispatch::new()
        .format(|out, message, record| match record.metadata().target() {
            _ => out.finish(format_args!(
                "[{} - {}] <{}> {}: {}",
                humantime::format_rfc3339(std::time::SystemTime::now()),
                record.level(),
                record.target(),
                record.file().unwrap_or("??"),
                message
            )),
        })
        .chain({
            fern::Dispatch::new()
                .level(CONFIG.with_borrow(|c| c.log.stdout_level.clone()).into())
                .chain(io::stdout())
        })
        .chain({
            fern::Dispatch::new()
                .level(CONFIG.with_borrow(|c| c.log.stdout_level.clone()).into())
                .chain(fern::log_file("./node.log")?)
        })
        .apply()?;

    Ok(())
}
